// (C) 2001-2019 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// synopsys translate_off
`timescale 1 ps / 1 ps
// synopsys translate_on

module desc_table_fetch  #
 (
    parameter DESC_ID  = 8'h80
  )
 (
      input logic                                   Clk_i,
      input logic                                   Rstn_i,

      /// DMA programming interface
      output   logic  [159:0]                       DmaTxData_o,
      output   logic                                DmaTxValid_o,
      input    logic                                DmaTxReady_i,

      input  logic [63:0]                           rc_dma_desc_addr_i,
      input  logic [63:0]                           ep_dma_desc_addr_i,
      input  logic [7:0]                            dma_lptr_i,
      input  logic [6:0]                            dma_lptr_plusone_i,
	  input  logic [6:0]                            dma_table_size_i,
	  input  logic                                  rollover_operation_i,                        
	  input  logic                                  fetch_dt_i,
      output logic                                  fetch_dt_ack_o
 );

localparam     DESC_ID_ROLLOVER = (DESC_ID == 8'h80) ? 8'hFE : 8'hFF;
logic [7:0]    num_desc; // change width to 8 for lining purpose
logic [6:0]    curr_pntr,next_current_ptr,prev_last_pntr,num_desc_reg,num_desc_reg1,subtract_tablesize;
logic          num_desc_msb,num_desc_msb_reg;
logic [7:0]    unique_dt_id;

logic [63:0]   rp_dt_addr;
logic [17:0]   length;
logic [159:0]  dt_descr;
logic          tx_valid;
logic          tx_valid_cmb;
logic          idle_state,cal_msb_state,cal_length_state,fetch_dt_state ;


//Start Processing when fetch is high
//Calculate number of descriptor in this state by simple subtraction.
always_ff @(posedge Clk_i) begin
  if (~Rstn_i)
    idle_state <= 1'b1;
  else
    idle_state <= (~fetch_dt_i & idle_state) | (fetch_dt_state & DmaTxReady_i);
end
//Calcluate Max Descriptor requested state
//As descriptor pointer holds value 1 less than actual descriptor it is pointing
//Subtraction needs to be adjusted when maximum descriptors are requested.
always_ff @(posedge Clk_i) begin
  if (~Rstn_i)
    cal_msb_state <= 1'b0;
  else
    cal_msb_state <= idle_state & fetch_dt_i;
end
//Calcluate length state
always_ff @(posedge Clk_i) begin
  if (~Rstn_i)
    cal_length_state <= 1'b0;
  else
    cal_length_state <= cal_msb_state;
end
//Fetch Descriptor Table state
always_ff @(posedge Clk_i) begin
  if (~Rstn_i)
    fetch_dt_state <= 1'b0;
  else
    fetch_dt_state <= cal_length_state | (fetch_dt_state & ~DmaTxReady_i);
end

assign fetch_dt_ack_o = idle_state & fetch_dt_i;
//Calculate length field of descriptor
//Length= num_descriptor * 32 = num_descriptor * 8 DW
//Store the previous last pointer to calculate number of descriptors to be fetched 
//Rollover to FF with rollover operation
always_ff @(posedge Clk_i) begin
  if (~Rstn_i)
    prev_last_pntr <= {7{1'b1}};
  else if (rollover_operation_i & idle_state)
    prev_last_pntr <= {7{1'b1}};
  else if (fetch_dt_i & idle_state)
    prev_last_pntr <= dma_lptr_i[6:0];
end
//Calculate number of descriptors 
//1. num descriptor = new last pointer - old last pointer + 1
//2. prev pointer starts from 7F always
//---if table size is 127 (actual descriptors are table size+1)
//-----if 7f is written then all 128 descriptor fetch is requested
//---if table size is less than 127
//-----if 7f is written then fetch diffrence between table size+1 - previous
//---Calculate MSB bit - 8
assign subtract_tablesize = rollover_operation_i ? dma_table_size_i : dma_lptr_i[6:0];
assign num_desc   = subtract_tablesize - prev_last_pntr;

always_ff @(posedge Clk_i) begin
 num_desc_reg  <= num_desc[6:0];
end

assign num_desc_msb = ~(|num_desc_reg);
always_ff @(posedge Clk_i) begin
 num_desc_msb_reg <= num_desc_msb;
end

always_ff @(posedge Clk_i) begin
 num_desc_reg1 <= num_desc_reg;
end
//2 clocks require to calculate number of descriptors/length
always_ff @(posedge Clk_i) begin
  if (cal_length_state)
   length[10:3] <= {num_desc_msb_reg,num_desc_reg1};
end
//Rollover to Zero with rollover operation 
//New current pointer is zero after rollover operation 
always_ff @(posedge Clk_i) begin
  if (rollover_operation_i)
    next_current_ptr <= {7{1'b0}};
  else if (idle_state)
    next_current_ptr <= dma_lptr_plusone_i[6:0];
end

assign length[2:0]   = {3{1'b0}};
assign length[17:11] = {7{1'b0}};

//Pre compute Current Pointer for next DT fetch

always_ff @(posedge Clk_i) begin
  if (~Rstn_i)
    curr_pntr <= {7{1'b0}};
  else if (tx_valid)
    curr_pntr <=  next_current_ptr ;
end

//RP Descriptor address calculation
//Descriptors are 32 byte aligned so last 5 bits are always zero

adder_64bit dt_addr
(
      .Clk_i    (Clk_i),
      .a_in     ({rc_dma_desc_addr_i[63:5],5'b00000}),
      .b_in     ({{52{1'b0}},curr_pntr[6:0],5'b00000}),
      .sum_out  (rp_dt_addr)

 );

//Descriptor ID Generation,Data Mover requires unique descriptor ID even for every descriptor Table fetch
always_ff @(posedge Clk_i) begin
  if (~Rstn_i)
    unique_dt_id <= DESC_ID;
  else begin
    if (DESC_ID_ROLLOVER == unique_dt_id)
	  unique_dt_id <= DESC_ID;
	else if (tx_valid) 
	  unique_dt_id <= unique_dt_id + 8'h02 ;
  end
end
assign dt_descr     = { 1'b0,unique_dt_id,length,ep_dma_desc_addr_i,rp_dt_addr };

//Assert Tx valid when not in idle state and Tx ready high
//Tready will be high for once in three clocks

assign tx_valid_cmb  = (fetch_dt_state) & DmaTxReady_i;

always_ff @(posedge Clk_i) begin
  tx_valid <= tx_valid_cmb;
end

assign DmaTxValid_o = tx_valid;
assign DmaTxData_o  = dt_descr;


endmodule
