// (C) 2001-2019 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// synopsys translate_off
`timescale 1 ps / 1 ps
// synopsys translate_on

module sync_fifo #(
      parameter  RAMSTYLE          = "MLAB",       //Type of RAM MLAB M20K
      parameter  AWIDTH            = 6,            //RAM Address Width + 1
      parameter  DEPTH             = 32,           //Fifo Depth
      parameter  DWIDTH            = 258,          //RAM data Width
      parameter  HOLDDOUT          = 0,            //DOUT will have last read value : Only applicable when NUM_PIPE > 1
      parameter  NUM_PIPES         = 2             //Number of o/p registers      
) 
 (
      input   logic                                Clk_i,
      input   logic                                Rstn_i,
      input   logic                                Wren_i,
      input   logic                                Rden_i,
      input   logic [DWIDTH-1:0]                   Data_i,
      
      output  logic [DWIDTH-1:0]                   Data_o,
      output  logic [DWIDTH-1:0]                   Data_outnoreg_o,
      output  logic                                Data_valid_o,
      output  logic                                Full_o,
      output  logic                                Empty_o,
      output  logic [AWIDTH-1 : 0]                 Occupancy_count_o,
      output  logic [AWIDTH-1 : 0]                 Available_count_o      
 );

(* ramstyle = RAMSTYLE *) logic [DWIDTH-1:0] ram_memory [DEPTH-1:0];
//Initialize memory with Zeroes
initial begin
  for (int i=0; i < DEPTH; i++ ) begin
    ram_memory[i] = {DWIDTH{1'b0}};
  end
end

logic [DWIDTH-1:0]                dout;
logic [DWIDTH*NUM_PIPES-1:0]      dout_reg;
logic [NUM_PIPES-1:0]             datavalid_reg;  /* synthesis maxfan = 16 */
logic [AWIDTH:0]                  rdptr,wrptr; // change width to AWIDTH + 1 for linting purpose
logic [AWIDTH-1:0]                wrptr_reg,rdptr_reg;
logic [AWIDTH-1:0]                wrptrold_reg,rdptrold_reg; 
logic                             full_reg,ptr_eq_wr,flag_eq_wr;
logic                             empty_reg,ptr_eq_rd,flag_eq_rd;
logic [AWIDTH-1:0]                available_count_reg;
logic [AWIDTH-1:0]                occupnacy_count_reg;

always @ (posedge Clk_i) begin
  if (Wren_i)
    ram_memory [wrptr_reg[AWIDTH-2:0]] <= Data_i ;  
end
assign dout = ram_memory[rdptr_reg[AWIDTH-2:0]];
//Pipe stages for data out
always_ff @ (posedge Clk_i) begin
  dout_reg [DWIDTH-1:0] <= dout ; 
end   
genvar i;
generate 
  for (i=1; i < NUM_PIPES; i=i+1 ) begin : data_pipe
    if ((i == NUM_PIPES-1) && (HOLDDOUT == 1)) begin
      always_ff @ (posedge Clk_i) begin
        if (datavalid_reg[NUM_PIPES-2])
          dout_reg [(DWIDTH*(i+1))-1:DWIDTH*i] <= dout_reg [(DWIDTH*i)-1:DWIDTH*(i-1)] ;  
      end   
    end
    else begin
      always_ff @ (posedge Clk_i) begin
        dout_reg [(DWIDTH*(i+1))-1:DWIDTH*i] <= dout_reg [(DWIDTH*i)-1:DWIDTH*(i-1)] ;  
      end
    end
  end
endgenerate
//Pipe stages for data valid
always_ff @ (posedge Clk_i) begin
  datavalid_reg[0] <= Rden_i ;  
end

generate 
  for (i=1; i < NUM_PIPES; i=i+1 ) begin : dvalid_pipe
    always_ff @ (posedge Clk_i) begin
      datavalid_reg [i] <= datavalid_reg [i-1] ;  
    end
  end
endgenerate

//Write pointer 
assign wrptr = wrptr_reg + Wren_i;
always_ff @ (posedge Clk_i) begin
  if (~Rstn_i)
    wrptr_reg <= {AWIDTH{1'b0}};
  else
    wrptr_reg <= wrptr[AWIDTH-1:0];
end
//Previous State of Write pointer 
always_ff @ (posedge Clk_i) begin
  if (~Rstn_i)
    wrptrold_reg <= {AWIDTH{1'b0}};
  else begin    
    wrptrold_reg <= wrptrold_reg;
    if (Wren_i)
      wrptrold_reg <= wrptr_reg;
  end
end
//Read pointer
assign rdptr = rdptr_reg + Rden_i;
always_ff @ (posedge Clk_i) begin
  if (~Rstn_i)
    rdptr_reg <= {AWIDTH{1'b0}};
  else
    rdptr_reg <= rdptr[AWIDTH-1:0];
end
//Previous state of read pointer
always_ff @ (posedge Clk_i) begin
  if (~Rstn_i)
    rdptrold_reg <= {AWIDTH{1'b1}}; 
  else begin    
    rdptrold_reg  <= rdptrold_reg;
    if (Rden_i)
      rdptrold_reg <= rdptr_reg;
  end
end 
//Full Generation
always_ff @ (posedge Clk_i) begin : full_flag 
  if (~Rstn_i)
    full_reg <= 1'b0;
  else begin
    full_reg <= full_reg;
    if (full_reg & Rden_i)
      full_reg <= 1'b0;
    if (ptr_eq_wr & ~flag_eq_wr & Wren_i & ~Rden_i)  
      full_reg <= 1'b1;
  end
end
//Compare with old Read Pointer
assign ptr_eq_wr  = (wrptr_reg[AWIDTH-2:0] == rdptrold_reg[AWIDTH-2:0]);
assign flag_eq_wr = (wrptr_reg[AWIDTH-1] == rdptrold_reg[AWIDTH-1]); 
//Empty Generation
always_ff @ (posedge Clk_i) begin : empty_flag 
  if (~Rstn_i)
    empty_reg <= 1'b1;
  else begin
    empty_reg <= empty_reg;
    if (empty_reg & Wren_i)
      empty_reg <= 1'b0;
    if (ptr_eq_rd & flag_eq_rd & ~Wren_i & Rden_i)  
      empty_reg <= 1'b1;
  end
end
//Compare with old Write Pointer
assign ptr_eq_rd  = (rdptr_reg[AWIDTH-2:0] == wrptrold_reg[AWIDTH-2:0]);
assign flag_eq_rd = (rdptr_reg[AWIDTH-1] == wrptrold_reg[AWIDTH-1]); 
//Compute occupancy counter
always_ff @ (posedge Clk_i) begin
  if (~Rstn_i)
    occupnacy_count_reg <= {AWIDTH{1'b0}};
  else begin
    occupnacy_count_reg <= occupnacy_count_reg;
    if (Wren_i & ~Rden_i)
     occupnacy_count_reg <= occupnacy_count_reg + 1'b1;
    if (~Wren_i & Rden_i)
     occupnacy_count_reg <= occupnacy_count_reg - 1'b1;    
  end    
end
//Compute Available locations
always_ff @ (posedge Clk_i) begin
  if (~Rstn_i)
    available_count_reg <= DEPTH;
  else begin
    available_count_reg <= available_count_reg;
    if (Wren_i & ~Rden_i)
     available_count_reg <= available_count_reg - 1'b1;
    if (~Wren_i & Rden_i)
     available_count_reg <= available_count_reg + 1'b1;    
  end    
end

assign Empty_o             = empty_reg;
assign Full_o              = full_reg;
assign Data_o              = dout_reg[(DWIDTH*NUM_PIPES)-1:DWIDTH*(NUM_PIPES-1)];
assign Data_outnoreg_o     = dout;
assign Data_valid_o        = datavalid_reg[NUM_PIPES-1];
assign Available_count_o   = available_count_reg;
assign Occupancy_count_o   = occupnacy_count_reg;

// Assertions
`ifdef ALTERA_PCIE_S10_ENABLE_ASSERTIONS
property overflow_check;
  @(posedge Clk_i) 
  disable iff (~Rstn_i)
   ((wrptr_reg[AWIDTH-1] != rdptr_reg[AWIDTH-1]) && (wrptr_reg[AWIDTH-2:0] == rdptr_reg[AWIDTH-2:0])) -> (~Wren_i);
endproperty
ERROR_fifo_overflow :
  assert property (overflow_check) 
  else $display("ERROR:Write Attempt with FIFO FULL Condition");

property underflow_check;
  @(posedge Clk_i) 
  disable iff (~Rstn_i)
   (wrptr_reg[AWIDTH-1:0] == rdptr_reg[AWIDTH-1:0]) -> (~Rden_i);
endproperty
ERROR_fifo_underflow :
  assert property (underflow_check)
  else $display("ERROR:Read Attempt from Empty FIFO Condition");
`endif
endmodule
